﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Linq;
//using System.Threading.Tasks;
using System.Windows.Forms;

namespace RCServo_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		RCServo openCh = null;
		formCleaner cleaner;
		RCServo servo = null;
		private ErrorEventBox errorBox;
		private DataGraphBox positionGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(RCServo ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			positionGraphBox = new DataGraphBox(this, "Position");
			voltageCombo.DataSource = typeof(RCServoVoltage).ToList();

			servo = openCh ?? openArgs.makeChannel<RCServo>();

			servo.Attach += servo_attach;
			servo.Detach += servo_detach;
			servo.Error += servo_error;
			servo.TargetPositionReached += servo_stopped;
			servo.VelocityChange += Servo_VelocityChange;
			servo.PositionChange += Servo_PositionChange;

			try {
				servo.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			servo.Attach -= servo_attach;
			servo.Detach -= servo_detach;
			servo.Error -= servo_error;
			servo.TargetPositionReached -= servo_stopped;
			servo.VelocityChange -= Servo_VelocityChange;
			servo.PositionChange -= Servo_PositionChange;
			servo.Close();
		}
		void servo_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			positionGraphBox.SetTitle(DataGraphBox.GenerateTitle("Position", (Phidget)servo));

			calibrationBox.Visible = true;
			controlsBox.Visible = true;
			positionEventTxt.Text = "Unknown";
			velocityEventTxt.Text = "Unknown";
			stoppedCheckBox.Checked = true;
			motorDataBox.Visible = true;

			initializeController();
		}

		private void initializeController() {

			minPWTxt.Text = servo.MinPulseWidth.ToString("F0");
			maxPWTxt.Text = servo.MaxPulseWidth.ToString("F0");
			minPos.Text = servo.MinPosition.ToString("F0");
			maxPos.Text = servo.MaxPosition.ToString("F0");

			//ui labels
			positionMinLbl.Text = minPos.Text;
			positionMaxLbl.Text = maxPos.Text;

			try {
				servo.SpeedRampingState = servo.SpeedRampingState;
				speedRampingCheckBox.Checked = servo.SpeedRampingState;
				speedRampingCheckBox.Visible = true;
			} catch (PhidgetException ex) {
				if (ex.ErrorCode == ErrorCode.Unsupported) {
					speedRampingCheckBox.Visible = false;
				} else
					errorBox.addMessage("Error initializing: " + ex.Message);
			}

			try {
				servo.Voltage = servo.Voltage;
				voltagePanel.Visible = true;
			} catch (PhidgetException ex) {
				if (ex.ErrorCode == ErrorCode.Unsupported) {
					voltagePanel.Visible = false;
				} else
					errorBox.addMessage("Error initializing: " + ex.Message);
			}

			try {
				int interval = servo.DataInterval;

				dataIntervalTrk.Visible = true;
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(servo.MinDataInterval, servo.MaxDataInterval);
				dataIntervalTrk.Value = servo.DataInterval;
			} catch (PhidgetException ex) {
				if (ex.ErrorCode == ErrorCode.Unsupported) {
					dataIntervalTrk.Visible = false;
				} else
					errorBox.addMessage("Error initializing: " + ex.Message);
			}

			try {
				double temp = servo.Acceleration;

				accelPanel.Visible = true;
				accelMinLbl.Text = servo.MinAcceleration.ToString("F0");
				accelMaxLbl.Text = servo.MaxAcceleration.ToString("F0");
				acceltrk.Value = (int)((servo.Acceleration - servo.MinAcceleration) / (servo.MaxAcceleration - servo.MinAcceleration) * 1000);
				acceltxt.Text = servo.Acceleration.ToString("F0");
			} catch (PhidgetException ex) {
				if (ex.ErrorCode == ErrorCode.Unsupported) {
					accelPanel.Visible = false;
				} else
					errorBox.addMessage("Error initializing: " + ex.Message);
			}

			try {
				double temp = servo.VelocityLimit;

				velocityPanel.Visible = true;
				velocityEventTxt.Visible = true;
				velocityEventLabel.Visible = true;

				velocityMinLbl.Text = servo.MinVelocityLimit.ToString("F0");
				velocityMaxLbl.Text = servo.MaxVelocityLimit.ToString("F0");
				velocitytrk.Value = (int)(servo.VelocityLimit / servo.MaxVelocityLimit * 1000);
				velocitytxt.Text = servo.VelocityLimit.ToString("F0");
			} catch (PhidgetException ex) {
				if (ex.ErrorCode == ErrorCode.Unsupported) {
					velocityPanel.Visible = false;
					velocityEventTxt.Visible = false;
					velocityEventLabel.Visible = false;
				} else
					errorBox.addMessage("Error initializing: " + ex.Message);
			}


			switch (servo.DeviceID) {
			case DeviceID.PN_1000:
			case DeviceID.PN_1001:
				stoppedCheckBox.Visible = false;
				positionGraphButton.Visible = true;
				break;
			case DeviceID.PN_RCC0004:
			case DeviceID.PN_1061:
			case DeviceID.PN_1066:
				positionGraphButton.Visible = true;
				break;
			case DeviceID.PN_RCC1000:
				positionGraphButton.Visible = false;
				break;
			}

			enBtn.Text = "Enable";
		}

		void servo_detach(object sender, Phidget22.Events.DetachEventArgs e) {

			phidgetInfoBox1.Clear();
			cleaner.clean();

			calibrationBox.Visible = false;
			controlsBox.Visible = false;
			speedRampingCheckBox.Visible = false;
			motorDataBox.Visible = false;
			dataIntervalTrk.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}

		void servo_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void servo_stopped(object sender, Phidget22.Events.RCServoTargetPositionReachedEventArgs e) {
			positionEventTxt.Text = e.Position.ToString("F0");
			stoppedCheckBox.Checked = true;
		}

		private void Servo_PositionChange(Object sender, RCServoPositionChangeEventArgs e) {
			positionEventTxt.Text = e.Position.ToString("F0");
			positionGraphBox.addData(e.Position, servo.DataInterval);
		}

		private void Servo_VelocityChange(Object sender, RCServoVelocityChangeEventArgs e) {
			velocityEventTxt.Text = e.Velocity.ToString("F0");
		}

		//enable or disable the motor
		private void enBtn_Click(object sender, EventArgs e) {
			//toggle enable
			try {
				if (servo.Engaged == false) {
					servo.Engaged = true;
					enBtn.Text = "Disable";
				} else if (servo.Engaged == true) {
					servo.Engaged = false;
					enBtn.Text = "Enable";
				}
			} catch (PhidgetException ex) { errorBox.addMessage("Error enabling: " + ex.Message); }
		}

		//change the acceleration
		private void acceltrk_Scroll(object sender, EventArgs e) {
			try {
				servo.Acceleration = ((double)acceltrk.Value / 1000.0 * (servo.MaxAcceleration - servo.MinAcceleration) + servo.MinAcceleration);
				acceltxt.Text = servo.Acceleration.ToString("F0");
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting acceleration: " + ex.Message);
			}
		}
		//change the velocity
		private void velocitytrk_Scroll(object sender, EventArgs e) {
			try {
				servo.VelocityLimit = ((double)velocitytrk.Value / 1000.0 * servo.MaxVelocityLimit);
				velocitytxt.Text = servo.VelocityLimit.ToString("F0");
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting velocity: " + ex.Message);
			}
		}
		//change the target position
		private void positiontrk_Scroll(object sender, EventArgs e) {
			try {
				if (servo.Engaged) {
					stoppedCheckBox.Checked = false;
				}
			} catch { }

			try {
				servo.TargetPosition = ((double)positiontrk.Value / 1000.0 * (servo.MaxPosition - servo.MinPosition)) + servo.MinPosition;
				positiontxt.Text = servo.TargetPosition.ToString("F0");
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting target position: " + ex.Message);
			}
		}

		private void settingsApply_Click(object sender, EventArgs e) {
			try {
				//servo calibration parameters
				servo.MinPulseWidth = Convert.ToInt32(minPWTxt.Text);
				servo.MaxPulseWidth = Convert.ToInt32(maxPWTxt.Text);
				servo.MinPosition = Convert.ToDouble(minPos.Text);
				servo.MaxPosition = Convert.ToDouble(maxPos.Text);

				initializeController();

				if (servo.Engaged)
					positiontxt.Text = servo.Position.ToString();

				switch (servo.DeviceID) {
				case DeviceID.PN_1066:
				case DeviceID.PN_1061:
				case DeviceID.PN_RCC0004:
					velocitytxt.Text = servo.VelocityLimit.ToString();
					acceltxt.Text = servo.Acceleration.ToString();
					break;
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting parameters: " + ex.Message);
			} catch (Exception ex) {
				errorBox.addMessage("Invalid parameter value: " + ex.Message);
			}
		}

		private void voltageCombo_SelectedIndexChanged(object sender, EventArgs e) {

			if (servo == null)
				return;

			try {
				servo.Voltage = (RCServoVoltage)Enum.Parse(typeof(RCServoVoltage), voltageCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting voltage: " + ex.Message);
			}
		}

		private void speedRampingCheckBox_CheckedChanged(object sender, EventArgs e) {
			try {
				servo.SpeedRampingState = speedRampingCheckBox.Checked;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting speed ramping state: " + ex.Message);
			}
		}

		private void updateGraphDataInterval() {
			try {
				positionGraphBox.DataInterval = (1000.0 / servo.DataRate);
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}

		private void positionGraphButton_Click(object sender, EventArgs e) {
			if (!positionGraphBox.Visible)
				positionGraphBox.Show();
		}

		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = servo.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = servo.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = servo.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = servo.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void dataIntervalTrk_ValueChange(object sender, EventArgs e) {

			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					double temp = dataIntervalTrk.Value;
					servo.DataInterval = (int)temp;
					if (servo.DataInterval < (int)temp) {
						while (servo.DataInterval != (int)temp) {
							temp++;
							servo.DataInterval = (int)temp;
						}
					} else {
						while (servo.DataInterval != (int)temp) {
							temp--;
							servo.DataInterval = (int)temp;
						}
					}

					dataIntervalTrk.TrueValue = servo.DataInterval;
					dataIntervalTrk.Value = servo.DataInterval;

					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
		}

		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {

			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}
	}
}